@@ -10,10 +10,19 @@ from inspect import isgenerator
 from inspect import isgeneratorfunction
 from inspect import ismethod
 from inspect import ismethoddescriptor
-from inspect import isroutine
 from inspect import ismodule
+from inspect import isroutine
 from logging import getLogger
 
+from .utils import basestring
+from .utils import force_bind
+from .utils import logf
+from .utils import make_method_matcher
+from .utils import mimic
+from .utils import PY2
+from .utils import PY3
+from .utils import Sentinel
+
 try:
     import __builtin__
 except ImportError:
@@ -24,15 +33,6 @@ try:
 except ImportError:
     ClassType = type
 
-from .utils import basestring
-from .utils import force_bind
-from .utils import make_method_matcher
-from .utils import mimic
-from .utils import PY2
-from .utils import PY3
-from .utils import logf
-from .utils import Sentinel
-
 
 __all__ = 'weave', 'Aspect', 'Proceed', 'Return', 'ALL_METHODS', 'NORMAL_METHODS'
 
@@ -303,6 +303,7 @@ def weave(target, aspects, **options):
             if not callable(obj):
                 raise ExpectedAdvice('%s must be an `Aspect` instance or a callable.' % obj)
     assert target, "Can't weave falsy value %r." % target
+    logdebug("weave (target=%s, aspects=%s, **options=%s)", target, aspects, options)
     if isinstance(target, (list, tuple)):
         return Rollback([
             weave(item, aspects, **options) for item in target
@@ -349,7 +350,6 @@ def weave(target, aspects, **options):
         else:
             return weave(obj, aspects, **options)
     name = getattr(target, '__name__', None)
-    logdebug("weave (target=%s, aspects=%s, name=%s, **options=%s)", target, aspects, name, options)
     if name and getattr(__builtin__, name, None) is target:
         return weave_module_function(__builtin__, target, aspects, **options)
     elif PY3 and ismethod(target):
@@ -524,7 +524,7 @@ def weave_class(klass, aspect, methods=NORMAL_METHODS, subclasses=True, lazy=Fal
     return entanglement
 
 
-def patch_module(module, name, replacement, original=UNSPECIFIED, aliases=True, location=None):
+def patch_module(module, name, replacement, original=UNSPECIFIED, aliases=True, location=None, **bogus_options):
     """
     Low-level attribute patcher.
 
@@ -536,6 +536,7 @@ def patch_module(module, name, replacement, original=UNSPECIFIED, aliases=True,
 
     :returns: An :obj:`aspectlib.Entanglement` object.
     """
+
     rollback = Rollback()
     seen = False
     original = getattr(module, name) if original is UNSPECIFIED else original
