@@ -3,15 +3,21 @@ from __future__ import print_function
 from pytest import raises
 
 from aspectlib import PY2
-from aspectlib.test import format_calls
+from aspectlib.test import _Binds
+from aspectlib.test import _format_calls
+from aspectlib.test import _Raises
+from aspectlib.test import _Returns
 from aspectlib.test import mock
+from aspectlib.test import OrderedDict
 from aspectlib.test import record
 from aspectlib.test import Story
 from aspectlib.test import StoryResultWrapper
-from aspectlib.test import Unexpected
-
+from aspectlib.utils import PY26
+from aspectlib.utils import repr_ex
 from test_pkg1.test_pkg2 import test_mod
 
+format_calls = lambda calls: ''.join(_format_calls(calls))
+
 
 def module_fun(a, b=2):
     pass
@@ -193,59 +199,68 @@ def test_record_not_iscalled_and_results():
 
 
 def test_story_empty_play_noproxy():
-    with Story(test_mod).replay(proxy=False, strict=False) as replay:
+    with Story(test_mod).replay(recurse_lock=True, proxy=False, strict=False) as replay:
         raises(AssertionError, test_mod.target)
 
-    assert replay._calls.actual == {}
+    assert replay._actual == {}
 
 
 def test_story_empty_play_proxy():
     assert test_mod.target() is None
     raises(TypeError, test_mod.target, 123)
 
-    with Story(test_mod).replay(proxy=True, strict=False) as replay:
+    with Story(test_mod).replay(recurse_lock=True, proxy=True, strict=False) as replay:
         assert test_mod.target() is None
         raises(TypeError, test_mod.target, 123)
 
-    assert format_calls(replay._calls.actual) == format_calls({
-        ('test_pkg1.test_pkg2.test_mod.target', (), frozenset([])): (
-            None, None
-        ),
-        ('test_pkg1.test_pkg2.test_mod.target', (123,), frozenset([])): (
-            None, TypeError('target() takes no arguments (1 given)'
-                            if PY2
-                            else 'target() takes 0 positional arguments but 1 was given',)
-        )
-    })
+    assert format_calls(replay._actual) == format_calls(OrderedDict([
+        ((None, 'test_pkg1.test_pkg2.test_mod.target', '', ''), _Returns("None")),
+        ((None, 'test_pkg1.test_pkg2.test_mod.target', '123', ''), _Raises(repr_ex(TypeError(
+            'target() takes no arguments (1 given)' if PY2 else
+            'target() takes 0 positional arguments but 1 was given',
+        ))))
+    ]))
 
 
 def test_story_empty_play_noproxy_class():
-    with Story(test_mod).replay(proxy=False, strict=False) as replay:
+    with Story(test_mod).replay(recurse_lock=True, proxy=False, strict=False) as replay:
         raises(AssertionError, test_mod.Stuff, 1, 2)
 
-    assert replay._calls.actual == {}
+    assert replay._actual == {}
+
+
+def test_story_empty_play_error_on_init():
+    with Story(test_mod).replay(strict=False) as replay:
+        raises(ValueError, test_mod.Stuff, "error")
+        print(replay._actual)
+    assert replay._actual == OrderedDict([
+        ((None, 'test_pkg1.test_pkg2.test_mod.Stuff', "'error'", ''), _Raises('ValueError()'))
+    ])
 
 
 def test_story_half_play_noproxy_class():
     with Story(test_mod) as story:
         obj = test_mod.Stuff(1, 2)
 
-    with story.replay(proxy=False, strict=False):
+    with story.replay(recurse_lock=True, proxy=False, strict=False):
         obj = test_mod.Stuff(1, 2)
         raises(AssertionError, obj.mix, 3, 4)
 
 
-def test_story_text_helpers():
+def test_xxx():
     with Story(test_mod) as story:
         obj = test_mod.Stuff(1, 2)
         test_mod.target(1) == 2
         test_mod.target(2) == 3
+        test_mod.target(3) ** ValueError
+        other = test_mod.Stuff(2, 2)
+        obj.other('a') == other
         obj.meth('a') == 'x'
-        obj.meth('b') == 'x'
         obj = test_mod.Stuff(2, 3)
+        obj.meth() ** ValueError('crappo')
         obj.meth('c') == 'x'
 
-    with story.replay(strict=False) as replay:
+    with story.replay(recurse_lock=True, strict=False) as replay:
         obj = test_mod.Stuff(1, 2)
         obj.meth('a')
         test_mod.target(1)
@@ -255,45 +270,73 @@ def test_story_text_helpers():
         obj = test_mod.Stuff(4, 4)
         obj.meth()
 
+    for k, v in story._calls.items():
+        print(k, "=>", v)
+    print("############## UNEXPECTED ##############")
+    for k, v in replay._actual.items():
+        print(k, "=>", v)
+
+    # TODO
+
+def test_story_text_helpers():
+    with Story(test_mod) as story:
+        obj = test_mod.Stuff(1, 2)
+        obj.meth('a') == 'x'
+        obj.meth('b') == 'y'
+        obj = test_mod.Stuff(2, 3)
+        obj.meth('c') == 'z'
+        test_mod.target(1) == 2
+        test_mod.target(2) == 3
+
+    with story.replay(recurse_lock=True, strict=False) as replay:
+        obj = test_mod.Stuff(1, 2)
+        obj.meth('a')
+        obj.meth()
+        obj = test_mod.Stuff(4, 4)
+        obj.meth()
+        test_mod.func(5)
+        test_mod.target(1)
+
     print (replay.missing())
-    assert replay.missing() == """stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)
-stuff_1.meth('b') == 'x'  # returns
-stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(2, 3)  # was never called !
-stuff_2.meth('c') == 'x'  # returns
+    assert replay.missing() == """stuff_1.meth('b') == 'y'  # returns
+stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(2, 3)
+stuff_2.meth('c') == 'z'  # returns
 test_pkg1.test_pkg2.test_mod.target(2) == 3  # returns
 """
     print (replay.unexpected())
-    assert replay.unexpected() == """stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)
-stuff_1.meth() == None  # returns
-stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(4, 4)  # was never called !
+    assert replay.unexpected() == """stuff_1.meth() == None  # returns
+stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(4, 4)
 stuff_2.meth() == None  # returns
 test_pkg1.test_pkg2.test_mod.func(5) == None  # returns
 """
     print (replay.diff())
-    assert replay.diff() == """--- expected
-+++ actual
-@@ -1,7 +1,7 @@
- stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)
- stuff_1.meth('a') == 'x'  # returns
--stuff_1.meth('b') == 'x'  # returns
--stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(2, 3)
--stuff_2.meth('c') == 'x'  # returns
-+stuff_1.meth() == None  # returns
-+stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(4, 4)  # was never called !
-+stuff_2.meth() == None  # returns
-+test_pkg1.test_pkg2.test_mod.func(5) == None  # returns
- test_pkg1.test_pkg2.test_mod.target(1) == 2  # returns
--test_pkg1.test_pkg2.test_mod.target(2) == 3  # returns
-""" or replay.diff() == """--- expected """ """
+    if PY26:
+        assert replay.diff() == """--- expected """ """
 +++ actual """ """
 @@ -1,7 +1,7 @@
  stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)
  stuff_1.meth('a') == 'x'  # returns
--stuff_1.meth('b') == 'x'  # returns
+-stuff_1.meth('b') == 'y'  # returns
 -stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(2, 3)
--stuff_2.meth('c') == 'x'  # returns
+-stuff_2.meth('c') == 'z'  # returns
 +stuff_1.meth() == None  # returns
-+stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(4, 4)  # was never called !
++stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(4, 4)
++stuff_2.meth() == None  # returns
++test_pkg1.test_pkg2.test_mod.func(5) == None  # returns
+ test_pkg1.test_pkg2.test_mod.target(1) == 2  # returns
+-test_pkg1.test_pkg2.test_mod.target(2) == 3  # returns
+"""
+    else:
+        assert replay.diff() == """--- expected
++++ actual
+@@ -1,7 +1,7 @@
+ stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)
+ stuff_1.meth('a') == 'x'  # returns
+-stuff_1.meth('b') == 'y'  # returns
+-stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(2, 3)
+-stuff_2.meth('c') == 'z'  # returns
++stuff_1.meth() == None  # returns
++stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(4, 4)
 +stuff_2.meth() == None  # returns
 +test_pkg1.test_pkg2.test_mod.func(5) == None  # returns
  test_pkg1.test_pkg2.test_mod.target(1) == 2  # returns
@@ -302,7 +345,7 @@ test_pkg1.test_pkg2.test_mod.func(5) == None  # returns
 
 
 def test_story_empty_play_proxy_class_missing_report():
-    with Story(test_mod).replay(proxy=True, strict=False) as replay:
+    with Story(test_mod).replay(recurse_lock=True, proxy=True, strict=False) as replay:
         obj = test_mod.Stuff(1, 2)
         obj.mix(3, 4)
         obj.mix('a', 'b')
@@ -315,60 +358,65 @@ def test_story_empty_play_proxy_class_missing_report():
         raises(ValueError, obj.raises, 123)
         test_mod.ThatLONGStuf(1).mix(2)
         test_mod.ThatLONGStuf(3).mix(4)
-        obj = test_mod.ThatLONGStuf(1)
+        obj = test_mod.ThatLONGStuf(2)
         obj.mix()
         obj.meth()
         obj.mix(10)
 
     print(repr(replay.diff()))
 
-    assert replay.diff() == """--- expected
-+++ actual
-@@ -0,0 +1,17 @@
-+stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(0, 1)  # was never called !
-+stuff_1.mix('a', 'b') == (0, 1, 'a', 'b')  # returns
-+stuff_1.mix(3, 4) == (0, 1, 3, 4)  # returns
-+stuff_1.raises(123) ** ValueError((123,))  # raises
-+stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)  # was never called !
-+stuff_2.mix('a', 'b') == (1, 2, 'a', 'b')  # returns
-+stuff_2.mix(3, 4) == (1, 2, 3, 4)  # returns
-+stuff_2.raises(123) ** ValueError((123,))  # raises
-+that_long_stuf_1 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(1)  # was never called !
-+that_long_stuf_1.meth() == None  # returns
-+that_long_stuf_1.mix() == (1,)  # returns
-+that_long_stuf_1.mix(10) == (1, 10)  # returns
-+that_long_stuf_1.mix(2) == (1, 2)  # returns
-+that_long_stuf_2 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(3)  # was never called !
-+that_long_stuf_2.mix(4) == (3, 4)  # returns
-+test_pkg1.test_pkg2.test_mod.raises('badarg') ** ValueError(('badarg',))  # raises
-+test_pkg1.test_pkg2.test_mod.target() == None  # returns
-""" or replay.diff() == """--- expected """ """
+    if PY26:
+        assert replay.diff() == """--- expected """ """
 +++ actual """ """
-@@ -1,0 +1,17 @@
-+stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(0, 1)  # was never called !
-+stuff_1.mix('a', 'b') == (0, 1, 'a', 'b')  # returns
-+stuff_1.mix(3, 4) == (0, 1, 3, 4)  # returns
-+stuff_1.raises(123) ** ValueError((123,))  # raises
-+stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)  # was never called !
-+stuff_2.mix('a', 'b') == (1, 2, 'a', 'b')  # returns
-+stuff_2.mix(3, 4) == (1, 2, 3, 4)  # returns
-+stuff_2.raises(123) ** ValueError((123,))  # raises
-+that_long_stuf_1 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(1)  # was never called !
-+that_long_stuf_1.meth() == None  # returns
-+that_long_stuf_1.mix() == (1,)  # returns
-+that_long_stuf_1.mix(10) == (1, 10)  # returns
-+that_long_stuf_1.mix(2) == (1, 2)  # returns
-+that_long_stuf_2 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(3)  # was never called !
-+that_long_stuf_2.mix(4) == (3, 4)  # returns
-+test_pkg1.test_pkg2.test_mod.raises('badarg') ** ValueError(('badarg',))  # raises
+@@ -1,0 +1,18 @@
++stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)
++stuff_1.mix(3, 4) == (1, 2, 3, 4)  # returns
++stuff_1.mix('a', 'b') == (1, 2, 'a', 'b')  # returns
++stuff_1.raises(123) ** ValueError((123,),)  # raises
++stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(0, 1)
++stuff_2.mix('a', 'b') == (0, 1, 'a', 'b')  # returns
++stuff_2.mix(3, 4) == (0, 1, 3, 4)  # returns
 +test_pkg1.test_pkg2.test_mod.target() == None  # returns
++test_pkg1.test_pkg2.test_mod.raises('badarg') ** ValueError(('badarg',),)  # raises
++stuff_2.raises(123) ** ValueError((123,),)  # raises
++that_long_stuf_1 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(1)
++that_long_stuf_1.mix(2) == (1, 2)  # returns
++that_long_stuf_2 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(3)
++that_long_stuf_2.mix(4) == (3, 4)  # returns
++that_long_stuf_3 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(2)
++that_long_stuf_3.mix() == (2,)  # returns
++that_long_stuf_3.meth() == None  # returns
++that_long_stuf_3.mix(10) == (2, 10)  # returns
+"""
+    else:
+        assert replay.diff() == """--- expected
++++ actual
+@@ -0,0 +1,18 @@
++stuff_1 = test_pkg1.test_pkg2.test_mod.Stuff(1, 2)
++stuff_1.mix(3, 4) == (1, 2, 3, 4)  # returns
++stuff_1.mix('a', 'b') == (1, 2, 'a', 'b')  # returns
++stuff_1.raises(123) ** ValueError((123,),)  # raises
++stuff_2 = test_pkg1.test_pkg2.test_mod.Stuff(0, 1)
++stuff_2.mix('a', 'b') == (0, 1, 'a', 'b')  # returns
++stuff_2.mix(3, 4) == (0, 1, 3, 4)  # returns
++test_pkg1.test_pkg2.test_mod.target() == None  # returns
++test_pkg1.test_pkg2.test_mod.raises('badarg') ** ValueError(('badarg',),)  # raises
++stuff_2.raises(123) ** ValueError((123,),)  # raises
++that_long_stuf_1 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(1)
++that_long_stuf_1.mix(2) == (1, 2)  # returns
++that_long_stuf_2 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(3)
++that_long_stuf_2.mix(4) == (3, 4)  # returns
++that_long_stuf_3 = test_pkg1.test_pkg2.test_mod.ThatLONGStuf(2)
++that_long_stuf_3.mix() == (2,)  # returns
++that_long_stuf_3.meth() == None  # returns
++that_long_stuf_3.mix(10) == (2, 10)  # returns
 """
 
 
 def test_story_empty_play_proxy_class():
     assert test_mod.Stuff(1, 2).mix(3, 4) == (1, 2, 3, 4)
 
-    with Story(test_mod).replay(proxy=True, strict=False) as replay:
+    with Story(test_mod).replay(recurse_lock=True, proxy=True, strict=False) as replay:
         obj = test_mod.Stuff(1, 2)
         assert obj.mix(3, 4) == (1, 2, 3, 4)
         assert obj.mix('a', 'b') == (1, 2, 'a', 'b')
@@ -381,22 +429,22 @@ def test_story_empty_play_proxy_class():
 
         raises(TypeError, obj.meth, 123)
 
-    assert format_calls(replay._calls.actual) == format_calls({
-        ('test_pkg1.test_pkg2.test_mod.Stuff', (1, 2), frozenset([])): Unexpected({
-            ('mix', ('a', 'b'), frozenset([])): ((1, 2, 'a', 'b'), None),
-            ('mix', (3, 4), frozenset([])): ((1, 2, 3, 4), None),
-            ('meth', (123,), frozenset([])): (None, TypeError('meth() takes exactly 1 argument (2 given)'
-                                                              if PY2
-                                                              else 'meth() takes 1 positional argument but 2 were given',))
-        }),
-        ('test_pkg1.test_pkg2.test_mod.Stuff', (0, 1), frozenset([])): Unexpected({
-            ('mix', ('a', 'b'), frozenset([])): ((0, 1, 'a', 'b'), None),
-            ('mix', (3, 4), frozenset([])): ((0, 1, 3, 4), None),
-            ('meth', (123,), frozenset([])): (None, TypeError('meth() takes exactly 1 argument (2 given)'
-                                                              if PY2
-                                                              else 'meth() takes 1 positional argument but 2 were given',))
-        })
-    })
+    assert format_calls(replay._actual) == format_calls(OrderedDict([
+        ((None, 'test_pkg1.test_pkg2.test_mod.Stuff', "1, 2", ''), _Binds('stuff_1')),
+        (('stuff_1', 'mix', "3, 4", ''), _Returns("(1, 2, 3, 4)")),
+        (('stuff_1', 'mix', "'a', 'b'", ''), _Returns("(1, 2, 'a', 'b')")),
+        (('stuff_1', 'meth', "123", ''), _Raises(repr_ex(TypeError(
+            'meth() takes exactly 1 argument (2 given)' if PY2 else
+            'meth() takes 1 positional argument but 2 were given'
+        )))),
+        ((None, 'test_pkg1.test_pkg2.test_mod.Stuff', "0, 1", ''), _Binds('stuff_2')),
+        (('stuff_2', 'mix', "'a', 'b'", ''), _Returns("(0, 1, 'a', 'b')")),
+        (('stuff_2', 'mix', "3, 4", ''), _Returns("(0, 1, 3, 4)")),
+        (('stuff_2', 'meth', "123", ''), _Raises(repr_ex(TypeError(
+            'meth() takes exactly 1 argument (2 given)' if PY2 else
+            'meth() takes 1 positional argument but 2 were given'
+        ))))
+    ]))
 
 
 def test_story_half_play_proxy_class():
@@ -406,7 +454,7 @@ def test_story_half_play_proxy_class():
         obj = test_mod.Stuff(1, 2)
         obj.mix(3, 4) == (1, 2, 3, 4)
 
-    with story.replay(proxy=True, strict=False) as replay:
+    with story.replay(recurse_lock=True, proxy=True, strict=False) as replay:
         obj = test_mod.Stuff(1, 2)
         assert obj.mix(3, 4) == (1, 2, 3, 4)
         assert obj.meth() is None
@@ -418,21 +466,20 @@ def test_story_half_play_proxy_class():
         assert obj.mix(3, 4) == (0, 1, 3, 4)
 
         raises(TypeError, obj.meth, 123)
-    assert replay.unexpected() == format_calls({
-        ('test_pkg1.test_pkg2.test_mod.Stuff', (1, 2), frozenset([])): {
-            ('meth', (), frozenset([])): (None, None),
-            ('meth', (123,), frozenset([])): (None, TypeError('meth() takes exactly 1 argument (2 given)'
-                                                              if PY2
-                                                              else 'meth() takes 1 positional argument but 2 were given',))
-        },
-        ('test_pkg1.test_pkg2.test_mod.Stuff', (0, 1), frozenset([])): Unexpected({
-            ('mix', ('a', 'b'), frozenset([])): ((0, 1, 'a', 'b'), None),
-            ('mix', (3, 4), frozenset([])): ((0, 1, 3, 4), None),
-            ('meth', (123,), frozenset([])): (None, TypeError('meth() takes exactly 1 argument (2 given)'
-                                                              if PY2
-                                                              else 'meth() takes 1 positional argument but 2 were given',))
-        })
-    })
+    assert replay.unexpected() == format_calls(OrderedDict([
+        (('stuff_1', 'meth', '', ''), _Returns('None')),
+        (('stuff_1', 'meth', '123', ''), _Raises(repr_ex(TypeError(
+            'meth() takes exactly 1 argument (2 given)' if PY2 else
+            'meth() takes 1 positional argument but 2 were given'
+        )))),
+        ((None, 'test_pkg1.test_pkg2.test_mod.Stuff', '0, 1', ''), _Binds("stuff_2")),
+        (('stuff_2', 'mix', "'a', 'b'", ''), _Returns("(0, 1, 'a', 'b')")),
+        (('stuff_2', 'mix',  '3, 4', ''), _Returns('(0, 1, 3, 4)')),
+        (('stuff_2', 'meth', '123', ''), _Raises(repr_ex(TypeError(
+            'meth() takes exactly 1 argument (2 given)' if PY2 else
+            'meth() takes 1 positional argument but 2 were given'
+        ))))
+    ]))
 
 
 def test_story_full_play_noproxy():
@@ -440,7 +487,7 @@ def test_story_full_play_noproxy():
         test_mod.target(123) == 'foobar'
         test_mod.target(1234) ** ValueError
 
-    with story.replay(proxy=False, strict=False, dump=False) as replay:
+    with story.replay(recurse_lock=True, proxy=False, strict=False, dump=False) as replay:
         raises(AssertionError, test_mod.target)
         assert test_mod.target(123) == 'foobar'
         raises(ValueError, test_mod.target, 1234)
@@ -453,7 +500,7 @@ def test_story_full_play_noproxy_dump():
         test_mod.target(123) == 'foobar'
         test_mod.target(1234) ** ValueError
 
-    with story.replay(proxy=False, strict=False, dump=True) as replay:
+    with story.replay(recurse_lock=True, proxy=False, strict=False, dump=True) as replay:
         raises(AssertionError, test_mod.target)
         assert test_mod.target(123) == 'foobar'
         raises(ValueError, test_mod.target, 1234)
@@ -466,22 +513,20 @@ def test_story_full_play_proxy():
         test_mod.target(123) == 'foobar'
         test_mod.target(1234) ** ValueError
 
-    with story.replay(proxy=True, strict=False) as replay:
+    with story.replay(recurse_lock=True, proxy=True, strict=False) as replay:
         assert test_mod.target() is None
         assert test_mod.target(123) == 'foobar'
         raises(ValueError, test_mod.target, 1234)
         raises(TypeError, test_mod.target, 'asdf')
 
-    assert replay.unexpected() == format_calls({
-        ('test_pkg1.test_pkg2.test_mod.target', (), frozenset([])): (
-            None, None
-        ),
-        ('test_pkg1.test_pkg2.test_mod.target', ('asdf',), frozenset([])): (
-            None, TypeError('target() takes no arguments (1 given)'
-                            if PY2
-                            else 'target() takes 0 positional arguments but 1 was given',)
-        )
-    })
+    assert replay.unexpected() == format_calls(OrderedDict([
+        ((None, 'test_pkg1.test_pkg2.test_mod.target', '', ''), _Returns("None")),
+        ((None, 'test_pkg1.test_pkg2.test_mod.target', "'asdf'", ''), _Raises(repr_ex(TypeError(
+            'target() takes no arguments (1 given)'
+            if PY2
+            else 'target() takes 0 positional arguments but 1 was given',)
+        )))
+    ]))
 
 
 def test_story_result_wrapper():
@@ -511,13 +556,28 @@ def test_story_create():
         assert isinstance(obj, test_mod.Stuff)
         obj.meth('other', 1, 2) == 123
         obj.mix('other') == 'mixymix'
-
-    assert story._calls == {
-        ('test_pkg1.test_pkg2.test_mod.Stuff', ('stuff',), frozenset()): {
-            ('meth', ('other', 1, 2), frozenset()): (123, None),
-            ('mix', ('other',), frozenset()): ('mixymix', None)
-        },
-        ('test_pkg1.test_pkg2.test_mod.target', (), frozenset()): (None, Exception),
-        ('test_pkg1.test_pkg2.test_mod.target', (1, 2, 3), frozenset()): ('foobar', None),
-        ('test_pkg1.test_pkg2.test_mod.target', ('a', 'b', 'c'), frozenset()): ('abc', None)
+    #from pprint import pprint as print
+    #print (dict(story._calls))
+    assert dict(story._calls) == {
+        (None, 'test_pkg1.test_pkg2.test_mod.Stuff',  "'stuff'", ''): _Binds('stuff_1'),
+        ('stuff_1', 'meth', "'other', 1, 2", ''): _Returns("123"),
+        ('stuff_1', 'mix', "'other'", ''): _Returns("'mixymix'"),
+        (None, 'test_pkg1.test_pkg2.test_mod.target', '', ''): _Raises("Exception"),
+        (None, 'test_pkg1.test_pkg2.test_mod.target', "1, 2, 3", ''): _Returns("'foobar'"),
+        (None, 'test_pkg1.test_pkg2.test_mod.target', "'a', 'b', 'c'", ''): _Returns("'abc'"),
     }
+
+def xtest_story_empty_play_proxy_class_dependencies():
+    with Story(test_mod).replay(recurse_lock=True, proxy=True, strict=False) as replay:
+        obj = test_mod.Stuff(1, 2)
+        other = obj.other('x')
+        raises(ValueError, other.raises, 'badarg')
+        other.mix(3, 4)
+        obj = test_mod.Stuff(0, 1)
+        obj.mix(3, 4)
+        other = obj.other(2)
+        other.mix(3, 4)
+
+    print(repr(replay.diff()))
+
+    assert replay.diff() == ""
