@@ -8,7 +8,8 @@ import asyncio
 from tornado import gen
 from tornado import ioloop
 
-from aspectlib import debug
+from aspectlib import debug, weave, ALL_METHODS
+from aspectlib.test import Story
 
 
 def test_decorate_asyncio_coroutine():
@@ -39,3 +40,17 @@ def test_decorate_tornado_coroutine():
     loop.run_sync(coro)
     output = buf.getvalue()
     assert 'coro => %r' % 'result' in output
+
+
+def test_mysql():
+    with Story(['MySQLdb.cursors.BaseCursor', 'MySQLdb.connections.Connection']) as story:
+        pass
+    rows = []
+    with story.replay(strict=False) as replay:
+        import MySQLdb
+        con = MySQLdb.connect('localhost', 'root', '')
+        con.select_db('mysql')
+        cursor = con.cursor()
+        cursor.execute('show tables')
+        rows.extend(cursor.fetchall())
+    assert '== (%s)' % ', '.join(repr(row) for row in rows) in replay.unexpected()
